@@ -1,5 +1,5 @@
 `createMatrix` <-
-function(noflevels, logical = FALSE) {
+function(noflevels) {
     conds <- length(noflevels)
     pwr <- unique(noflevels)
     if (any(pwr > 2)) {
@@ -19,9 +19,8 @@ function(noflevels, logical = FALSE) {
            rep.int(rep.int(seq_len(noflevels[x]) - 1, rep.int(mbase[x], noflevels[x])), orep[x])
         })
     }
-    if (logical) {
-        retmat <- matrix(as.logical(retmat), nrow=nrow(retmat), ncol=ncol(retmat))
+    if (is.vector(retmat)) {
+        retmat <- matrix(retmat, nrow=1)
     }
     return(retmat)
 }
-
